/*
 * matriu.c
 *      Matriu de reals.
 */

#include <assert.h>
#include <stdlib.h>

#include "matriu.h"

void InterReal(float *const a, float *const b);
void InterApReal(float **const a, float **const b);

/* Implementacio de les operacions */

int
FilesMatriu(Matriu m)
{
    return m.nf;
}

int
ColsMatriu(Matriu m)
{
    return m.nc;
}

Matriu
CrearMatriu(int nf, int nc)
{
    Matriu m;
    int i;

    assert(nf > 0 && nc > 0);

    m.nf = nf;
    m.nc = nc;
    m.telem = malloc(nf * sizeof (float *));
    assert(m.telem != NULL);

    i = 0;
    while (i != nf) {
      m.telem[i] = malloc(nc * sizeof (float));
      assert(m.telem[i] != NULL);
      i = i + 1;
    }
    return m;
}

void
DestruirMatriu(Matriu * const m)
{
    int i;

    i = 0;
    while (i != (*m).nf) {
      free((*m).telem[i]);
      i = i + 1;
    }
    free((*m).telem);
}

void
AssigMatriu(Matriu *const m, int i, int j,
            float f)
{
    assert(0 <= i && i < (*m).nf);
    assert(0 <= j && j < (*m).nc);

    (*m).telem[i][j] = f;
}

float
ConsMatriu(Matriu m, int i, int j)
{
    assert(0 <= i && i < m.nf);
    assert(0 <= j && j < m.nc);

    return m.telem[i][j];
}

void
IntFilesMatriu(Matriu * const m, int i, int j)
{
  assert(0 <= i && i < (*m).nf);
  assert(0 <= j && j < (*m).nc);

  InterApReal(&(*m).telem[i], &(*m).telem[j]);
}

void
IntColsMatriu(Matriu * const m, int i, int j)
{
  int k;

  assert(0 <= i && i < (*m).nf);
  assert(0 <= j && j < (*m).nc);

  k = 0;
  while (k != (*m).nf) {
    InterReal(&(*m).telem[k][i],
              &(*m).telem[k][j]);
    k = k + 1;
  }
}

void InterReal(float *const a, float *const b)
{
  float tmp;

  tmp = *a;
  *a = *b;
  *b = tmp;
}

void InterApReal(float **const a, float **const b)
{
  float *tmp;

  tmp = *a;
  *a = *b;
  *b = tmp;
}
